/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include "grlib.h"
#include "radioButton.h"
#include "checkbox.h"
#include "LcdDriver/Sharp96x96.h"
#include "images/images.h"
#include "driverlib.h"
#include "stdint.h"
#include "tmp-decode.h"
#include "HDC2010.h"

#define GPIO_PIN_ALL 0xFFFF

void clockInit(void);
void boardInit();
void timerInit();

void I2CInit();
void I2CSetAddr(char addr);
void I2C_Tx(char* tx_data, char tx_size);
void I2C_Rx(char* rx_data, char rx_size);

Graphics_Context g_sContext;
uint8_t g_delayCounter = 0;

void main(void)
{
    // disable watchdog and run all hardware init
    WDTCTL = WDTPW + WDTHOLD;
    boardInit();
    clockInit();
    timerInit();
    I2CInit();
    I2CSetAddr(0x40);
    Sharp96x96_initDisplay();

    /*
     * Disable the GPIO power-on default high-impedance mode to activate
     * previously configured port settings
     */
    PMM_unlockLPM5();

    // init GRLIB
    Graphics_initContext(&g_sContext, &g_sharp96x96LCD);
    Graphics_setForegroundColor(&g_sContext, ClrBlack);
    Graphics_setBackgroundColor(&g_sContext, ClrWhite);
    Graphics_setFont(&g_sContext, &g_sFontCmss20);
    Graphics_clearDisplay(&g_sContext);

    // Draw splash screen in display buffer
    Graphics_drawImage(&g_sContext, &TI_Logo_69x64_1BPP_UNCOMP, 30, 15);
    Graphics_drawStringCentered(&g_sContext,"HDC2010",AUTO_STRING_LENGTH,64,100,TRANSPARENT_TEXT);
    // write display buffer to display using SPI
    Graphics_flushBuffer(&g_sContext);
    // go to sleep (wake up from TIMER A after 1s)
    __bis_SR_register(LPM4_bits + GIE);

    char cf = 0;
    char data[] = {0, 1, 2, 3};
    char* stmp;
    char* shm;
    while(1){
        // check button S1
        if (!GPIO_getInputPinValue(GPIO_PORT_P3,GPIO_PIN0))
            cf = cf ? 0 : 1;
        // write to CONFIG register
        data[0] = HDC2010_MEAS_CONFIG;
        data[1] = 1;
        I2C_Tx(data,2);
        // re-configure timer for 1/4s
        Timer_A_clear(TIMER_A1_BASE);
        Timer_A_setCompareValue(TIMER_A1_BASE,TIMER_A_CAPTURECOMPARE_REGISTER_0,(uint16_t) 1<<8);
        // go to sleep
        __bis_SR_register(LPM4_bits + GIE);
        // re-configure timer for 1s
        Timer_A_setCompareValue(TIMER_A1_BASE,TIMER_A_CAPTURECOMPARE_REGISTER_0,(uint16_t) 1<<15);
        // set register pointer to 0
        data[0] = 0;
        I2C_Tx(data,1);
        // read 4 bytes of Temp + Hum result
        I2C_Rx(data,4);
        // convert results to strings
        stmp = tempstring(data[1],data[0],cf);
        shm = humstring(data[3],data[2]);
        // draw in display buffer
        Graphics_clearDisplay(&g_sContext);
        Graphics_drawImage(&g_sContext, &thermo11BPP_UNCOMP, 0, 0);
        Graphics_drawImage(&g_sContext, &humidity11BPP_UNCOMP, 0, 40);
        if (cf) {
            Graphics_drawPixel(&g_sContext,15-1,35-1);
            Graphics_drawPixel(&g_sContext,16-1,35-1);
        }
        else
            Graphics_drawPixel(&g_sContext,15-1,33-1);
        Graphics_setFont(&g_sContext, &g_sFontCmss20);
        Graphics_drawStringCentered(&g_sContext,"HDC2010",AUTO_STRING_LENGTH,64,100,TRANSPARENT_TEXT);
        Graphics_setFont(&g_sContext, &g_sFontCmss44i);
        Graphics_drawString(&g_sContext,stmp,6,10,6,TRANSPARENT_TEXT);
        Graphics_drawString(&g_sContext,shm,6,10,46,TRANSPARENT_TEXT);
        // write display buffer to display using SPI
        Graphics_flushBuffer(&g_sContext);
        // go to sleep
        __bis_SR_register(LPM4_bits + GIE);
        // do it again (2s refresh rate)
        __bis_SR_register(LPM4_bits + GIE);
    }
}

void clockInit(void)
{
    //0,3 = 4MHz
    //1,3 = 8MHz
    //1,4 = 16MHz
    //1,6 = 24MHz
    CS_setDCOFreq(CS_DCORSEL_1,CS_DCOFSEL_3);
    CS_initClockSignal(CS_ACLK,CS_LFXTCLK_SELECT,CS_CLOCK_DIVIDER_1);
    CS_initClockSignal(CS_SMCLK,CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
    CS_initClockSignal( CS_MCLK,CS_DCOCLK_SELECT,CS_CLOCK_DIVIDER_1);
}

void boardInit()
{
    GPIO_setAsOutputPin(GPIO_PORT_PA, GPIO_PIN_ALL);
    GPIO_setAsOutputPin(GPIO_PORT_PB, GPIO_PIN_ALL);
    GPIO_setOutputLowOnPin(GPIO_PORT_PA, GPIO_PIN_ALL);
    GPIO_setOutputLowOnPin(GPIO_PORT_PB, GPIO_PIN_ALL);
    GPIO_setAsInputPinWithPullUpResistor(GPIO_PORT_P3,GPIO_PIN0 + GPIO_PIN1);
}

void timerInit()
{
    Timer_A_initUpModeParam timerAInputParam =
    {
     TIMER_A_CLOCKSOURCE_ACLK,
     TIMER_A_CLOCKSOURCE_DIVIDER_1,
     32768,
     TIMER_A_TAIE_INTERRUPT_DISABLE,
     TIMER_A_CCIE_CCR0_INTERRUPT_ENABLE,
     TIMER_A_DO_CLEAR,
     1
    };

    Timer_A_initUpMode(TIMER_A1_BASE,&timerAInputParam);
}

#pragma vector=TIMER1_A0_VECTOR
__interrupt void TIMER1_A0_ISR(void)
{
//    Sharp96x96_SendToggleVCOMCommand();
    __bic_SR_register_on_exit(LPM0_bits);
}

char* i2c_data;
char i2c_size;
char i2c_count;
char tx =0,rx =0;

void I2CInit(){
    GPIO_setAsPeripheralModuleFunctionInputPin(
            GPIO_PORT_P1,
            GPIO_PIN6 + GPIO_PIN7,
            GPIO_SECONDARY_MODULE_FUNCTION
    );

    EUSCI_B_I2C_initMasterParam param = {0};
    param.selectClockSource = EUSCI_B_I2C_CLOCKSOURCE_SMCLK;
    param.i2cClk = CS_getSMCLK();
    param.dataRate = EUSCI_B_I2C_SET_DATA_RATE_400KBPS;
    param.byteCounterThreshold = 0;
    param.autoSTOPGeneration = EUSCI_B_I2C_NO_AUTO_STOP;
    EUSCI_B_I2C_initMaster(EUSCI_B0_BASE, &param);

    //Enable I2C Module to start operations
    EUSCI_B_I2C_enable(EUSCI_B0_BASE);
}

void I2CSetAddr(char addr){
    //Specify slave address
    EUSCI_B_I2C_setSlaveAddress(EUSCI_B0_BASE, addr);
}


void I2C_Tx(char* tx_data, char tx_size){
    i2c_data = tx_data;
    i2c_size = tx_size;
    i2c_count = 0;
    //Set Master in transmit mode
    EUSCI_B_I2C_setMode(EUSCI_B0_BASE, EUSCI_B_I2C_TRANSMIT_MODE);
    EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0 + EUSCI_B_I2C_NAK_INTERRUPT);
    EUSCI_B_I2C_enableInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0 + EUSCI_B_I2C_NAK_INTERRUPT);
    EUSCI_B_I2C_masterSendMultiByteStart(EUSCI_B0_BASE,i2c_data[i2c_count]);
    __bis_SR_register(CPUOFF + GIE);    // Enter LPM0 w/ interrupt
}

void I2C_Rx(char* rx_data, char rx_size){
    i2c_data = rx_data;
    i2c_size = rx_size;
    i2c_count = 0;
    //Set Master in receive mode
    EUSCI_B_I2C_setMode(EUSCI_B0_BASE, EUSCI_B_I2C_RECEIVE_MODE);
    EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_RECEIVE_INTERRUPT0 + EUSCI_B_I2C_NAK_INTERRUPT);
    EUSCI_B_I2C_enableInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_RECEIVE_INTERRUPT0 + EUSCI_B_I2C_NAK_INTERRUPT);
    if (rx_size == 1)
        i2c_data[0] = EUSCI_B_I2C_masterReceiveSingleByte(EUSCI_B0_BASE);
        else {
            EUSCI_B_I2C_masterReceiveStart(EUSCI_B0_BASE);
            __bis_SR_register(CPUOFF + GIE);    // Enter LPM0 w/ interrupt
        }
}
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=USCI_B0_VECTOR
__interrupt
#elif defined(__GNUC__)
__attribute__((interrupt(USCI_B0_VECTOR)))
#endif
void USCIB0_ISR(void)
{
    switch(__even_in_range(UCB0IV,0x1E))
    {
    case 0x00: break;       // Vector 0: No interrupts
    case 0x02: break;       // Vector 2: ALIFG
    case 0x04: break;       // Vector 4: NACKIFG
    case 0x06: break;       // Vector 6: STT IFG
    case 0x08: break;       // Vector 8: STPIFG
    case 0x0a: break;       // Vector 10: RXIFG3
    case 0x0c: break;       // Vector 14: TXIFG3
    case 0x0e: break;       // Vector 16: RXIFG2
    case 0x10: break;       // Vector 18: TXIFG2
    case 0x12: break;       // Vector 20: RXIFG1
    case 0x14: break;       // Vector 22: TXIFG1
    case 0x16:              // Vector 24: RXIFG0 aka USCI_I2C_UCRXIFG0
        rx++;
        char remain;
        i2c_data[i2c_count] = EUSCI_B_I2C_masterReceiveSingle(EUSCI_B0_BASE);
        remain = i2c_size - ++i2c_count;
        if(remain)
        {
            if (remain == 1){
                //Send stop condition.
                HWREG16(EUSCI_B0_BASE + OFS_UCBxCTLW0) |= UCTXSTP;
            }
        }
        else
        {
            EUSCI_B_I2C_disableInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_RECEIVE_INTERRUPT0);
            EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_RECEIVE_INTERRUPT0);
            __bic_SR_register_on_exit(CPUOFF);               // Exit LPM0
        }
        break;
    case 0x18:              // Vector 26: TXIFG0
        tx++;
        if(++i2c_count < i2c_size)
        {
            EUSCI_B_I2C_masterSendMultiByteNext(EUSCI_B0_BASE,i2c_data[i2c_count]);
        }
        else
        {
            EUSCI_B_I2C_masterSendMultiByteStop(EUSCI_B0_BASE);
            EUSCI_B_I2C_disableInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0);
            EUSCI_B_I2C_clearInterrupt(EUSCI_B0_BASE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0);
            __bic_SR_register_on_exit(CPUOFF);    // Exit LPM0
        }
        break;
    case 0x1a: break;       // Vector 28: BCNTIFG
    case 0x1c: break;       // Vector 30: clock low timeout
    case 0x1e: break;       // Vector 32: 9th bit
    default: break;
    }

}
